@@ -64,14 +64,6 @@
 #endif
 #endif
 
-#ifdef Q_OS_MAC
-#include <CoreFoundation/CFBundle.h>
-#include <CoreFoundation/CFString.h>
-#include <CoreFoundation/CFURL.h>
-#include <crt_externs.h> // for _NSGetArgc and friends
-#include <mach-o/dyld.h> // for _NSGetExecutablePath
-#endif
-
 #include <kinit_version.h>
 
 #include "klauncher_cmds.h"
@@ -80,33 +72,41 @@
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 #include <fixx11h.h>
-#include <kstartupinfo.h>
 #endif
+
+#if HAVE_XCB
+#include <NETWM>
+#include <xcb/xcb.h>
+#endif
+
+#include <kstartupinfo.h>
+
 #include <qstandardpaths.h>
 
+#include "kinit.h"
+#ifdef Q_OS_OSX
+#include "kinit_mac.h"
+#endif
+
 #ifdef Q_OS_UNIX
-#ifdef Q_OS_MAC
+//TODO: make sure what libraries we want here...
 static const char *extra_libs[] = {
+#ifdef Q_OS_OSX
     "libKF5KIOCore.5.dylib",
     "libKF5Parts.5.dylib",
     "libKF5Plasma.5.dylib"
-};
 #elif defined(Q_OS_CYGWIN)
-static const char *extra_libs[] = {
     "cygKF5KIOCore-5.dll",
     "cygKF5Parts-5.dll",
     "cygKF5Plasma-5.dll"
-};
 #else
-//TODO: make sure what libraries we want here...
-static const char *extra_libs[] = {
     "libKF5KIOCore.so.5",
     "libKF5Parts.so.5",
 //#ifdef __KDE_HAVE_GCC_VISIBILITY // Removed for KF5, we'll see.
     "libKF5Plasma.so.5"
 //#endif
-};
 #endif
+};
 #endif
 
 // #define SKIP_PROCTITLE 1
@@ -120,54 +120,29 @@ extern char **environ;
 
 #if HAVE_X11
 static int X11fd = -1;
-static Display *X11display = 0;
+static Display *X11display = nullptr;
 static int X11_startup_notify_fd = -1;
-static Display *X11_startup_notify_display = 0;
 #endif
-#define MAX_SOCK_FILE 255
+#if HAVE_XCB
+static xcb_connection_t *s_startup_notify_connection = nullptr;
+static int s_startup_notify_screen = 0;
+#endif
+// Finds size of sun_path without allocating a sockaddr_un to do it.
+// Assumes sun_path is at end of sockaddr_un though
+#define MAX_SOCK_FILE (sizeof(struct sockaddr_un) - offsetof(struct sockaddr_un,sun_path))
 static char sock_file[MAX_SOCK_FILE];
 
-static QByteArray displayEnvVarName()
+#if HAVE_X11 || HAVE_XCB
+static const char* displayEnvVarName_c()
 {
     // Can't use QGuiApplication::platformName() here, there is no app instance.
-#if HAVE_X11
     return "DISPLAY";
-#elif defined(Q_OS_MACX)
-    return "MAC_DISPLAY";
-#elif defined(Q_OS_WIN)
-    return "WIN_DISPLAY";
-#endif
 }
-
-/* Group data */
-static struct {
-    int maxname;
-    int fd[2];
-    int launcher[2]; /* socket pair for launcher communication */
-    int deadpipe[2]; /* pipe used to detect dead children */
-    int initpipe[2];
-    int wrapper; /* socket for wrapper communication */
-    int accepted_fd; /* socket accepted and that must be closed in the child process */
-    char result;
-    int exit_status;
-    pid_t fork;
-    pid_t launcher_pid;
-    pid_t kded_pid;
-    int n;
-    char **argv;
-    int (*func)(int, char *[]);
-    int (*launcher_func)(int);
-    bool debug_wait;
-    QByteArray errorMsg;
-    bool launcher_ok;
-    bool suicide;
-} d;
-
-struct child {
-    pid_t pid;
-    int sock; /* fd to write message when child is dead*/
-    struct child *next;
-};
+static inline QByteArray displayEnvVarName()
+{
+    return QByteArray::fromRawData(displayEnvVarName_c(), strlen(displayEnvVarName_c()));
+}
+#endif
 
 static struct child *children;
 
@@ -208,7 +183,7 @@ static void cleanup_fds()
  * Close fd's which are only useful for the parent process.
  * Restore default signal handlers.
  */
-static void close_fds()
+void close_fds()
 {
     while (struct child *child = children) {
         close(child->sock);
@@ -290,22 +265,9 @@ static void child_died(pid_t exit_pid, int exit_status)
     }
 }
 
-static void exitWithErrorMsg(const QString &errorMsg)
+void setup_tty(const char *tty)
 {
-    fprintf(stderr, "%s\n", errorMsg.toLocal8Bit().data());
-    QByteArray utf8ErrorMsg = errorMsg.toUtf8();
-    d.result = 3; // Error with msg
-    write(d.fd[1], &d.result, 1);
-    int l = utf8ErrorMsg.length();
-    write(d.fd[1], &l, sizeof(int));
-    write(d.fd[1], utf8ErrorMsg.data(), l);
-    close(d.fd[1]);
-    exit(255);
-}
-
-static void setup_tty(const char *tty)
-{
-    if (tty == NULL || *tty == '\0') {
+    if (tty == nullptr || *tty == '\0') {
         return;
     }
     QFile f(QFile::decodeName(tty));
@@ -325,30 +287,6 @@ static void setup_tty(const char *tty)
     close(fd);
 }
 
-#if HAVE_X11 // Only X11 supports multiple desktops
-// from kdecore/netwm.cpp
-static int get_current_desktop(Display *disp)
-{
-    int desktop = 0; // no desktop by default
-    Atom net_current_desktop = XInternAtom(disp, "_NET_CURRENT_DESKTOP", False);
-    Atom type_ret;
-    int format_ret;
-    unsigned char *data_ret;
-    unsigned long nitems_ret, unused;
-    if (XGetWindowProperty(disp, DefaultRootWindow(disp), net_current_desktop,
-                           0l, 1l, False, XA_CARDINAL, &type_ret, &format_ret, &nitems_ret, &unused, &data_ret)
-            == Success) {
-        if (type_ret == XA_CARDINAL && format_ret == 32 && nitems_ret == 1) {
-            desktop = *((long *) data_ret) + 1;
-        }
-        if (data_ret) {
-            XFree((char *) data_ret);
-        }
-    }
-    return desktop;
-}
-#endif
-
 // var has to be e.g. "DISPLAY=", i.e. with =
 const char *get_env_var(const char *var, int envc, const char *envs)
 {
@@ -366,10 +304,10 @@ const char *get_env_var(const char *var, int envc, const char *envs)
             env_l++;
         }
     }
-    return NULL;
+    return nullptr;
 }
 
-#if HAVE_X11
+#if HAVE_XCB
 static void init_startup_info(KStartupInfoId &id, const QByteArray &bin,
                               int envc, const char *envs)
 {
@@ -377,34 +315,40 @@ static void init_startup_info(KStartupInfoId &id, const QByteArray &bin,
     const char *dpy = get_env_var(envname.constData(), envc, envs);
     // this may be called in a child, so it can't use display open using X11display
     // also needed for multihead
-    X11_startup_notify_display = XOpenDisplay(dpy);
-    if (X11_startup_notify_display == NULL) {
+    s_startup_notify_connection = xcb_connect(dpy, &s_startup_notify_screen);
+    if (!s_startup_notify_connection) {
         return;
     }
-    X11_startup_notify_fd = XConnectionNumber(X11_startup_notify_display);
+    if (xcb_connection_has_error(s_startup_notify_connection)) {
+        xcb_disconnect(s_startup_notify_connection);
+        s_startup_notify_connection = nullptr;
+        return;
+    }
+    X11_startup_notify_fd = xcb_get_file_descriptor(s_startup_notify_connection);
+    NETRootInfo rootInfo(s_startup_notify_connection, NET::CurrentDesktop);
     KStartupInfoData data;
-    int desktop = get_current_desktop(X11_startup_notify_display);
-    data.setDesktop(desktop);
+    data.setDesktop(rootInfo.currentDesktop(true));
     data.setBin(QFile::decodeName(bin));
-    KStartupInfo::sendChangeX(X11_startup_notify_display, id, data);
-    XFlush(X11_startup_notify_display);
+    KStartupInfo::sendChangeXcb(s_startup_notify_connection, s_startup_notify_screen, id, data);
+    xcb_flush(s_startup_notify_connection);
 }
 
 static void complete_startup_info(KStartupInfoId &id, pid_t pid)
 {
-    if (X11_startup_notify_display == NULL) {
+    if (!s_startup_notify_connection) {
         return;
     }
     if (pid == 0) { // failure
-        KStartupInfo::sendFinishX(X11_startup_notify_display, id);
+        KStartupInfo::sendFinishXcb(s_startup_notify_connection, s_startup_notify_screen, id);
     } else {
         KStartupInfoData data;
         data.addPid(pid);
         data.setHostname();
-        KStartupInfo::sendChangeX(X11_startup_notify_display, id, data);
+        KStartupInfo::sendChangeXcb(s_startup_notify_connection, s_startup_notify_screen, id, data);
     }
-    XCloseDisplay(X11_startup_notify_display);
-    X11_startup_notify_display = NULL;
+    xcb_disconnect(s_startup_notify_connection);
+    s_startup_notify_connection = nullptr;
+    s_startup_notify_screen = 0;
     X11_startup_notify_fd = -1;
 }
 #endif
@@ -412,10 +356,10 @@ static void complete_startup_info(KStartupInfoId &id, pid_t pid)
 QByteArray execpath_avoid_loops(const QByteArray &exec, int envc, const char *envs, bool avoid_loops)
 {
     QStringList paths;
-    const QRegExp pathSepRegExp(QString::fromLatin1("[:\b]"));
+    const QRegExp pathSepRegExp(QStringLiteral("[:\b]"));
     if (envc > 0) { /* use the passed environment */
         const char *path = get_env_var("PATH=", envc, envs);
-        if (path != NULL) {
+        if (path != nullptr) {
             paths = QFile::decodeName(path).split(pathSepRegExp);
         }
     } else {
@@ -444,7 +388,7 @@ static void oom_protect_sighandler(int)
 {
 }
 
-static void reset_oom_protect()
+void reset_oom_protect()
 {
     if (oom_pipe <= 0) {
         return;
@@ -460,27 +404,56 @@ static void reset_oom_protect()
     sigprocmask(SIG_BLOCK, &sigs, &oldsigs);
     pid_t pid = getpid();
     if (write(oom_pipe, &pid, sizeof(pid_t)) > 0) {
-        sigsuspend(&oldsigs);   // wait for the signal to come
+        struct timespec timeout;
+        timeout.tv_sec = 1;
+        timeout.tv_nsec = 0;
+        do {
+            if (sigtimedwait(&sigs, nullptr, &timeout) < 0) { // wait for the signal to come
+                if (errno == EINTR) {
+                    // other signal
+                    continue;
+                } else if (errno == EAGAIN) {
+                    fprintf(stderr, "Timed out during reset OOM protection: %d\n", pid);
+                } else {
+                    fprintf(stderr, "Error during reset OOM protection: %d\n", pid);
+                }
+            }
+            break;
+        } while (true);
     } else {
 #ifndef NDEBUG
         fprintf(stderr, "Failed to reset OOM protection: %d\n", pid);
 #endif
     }
-    sigprocmask(SIG_SETMASK, &oldsigs, NULL);
-    sigaction(SIGUSR1, &oldact, NULL);
+    sigprocmask(SIG_SETMASK, &oldsigs, nullptr);
+    sigaction(SIGUSR1, &oldact, nullptr);
     close(oom_pipe);
     oom_pipe = -1;
 }
 #else
-static void reset_oom_protect()
+void reset_oom_protect()
 {
 }
 #endif
 
+#ifndef Q_OS_OSX
+static void exitWithErrorMsg(const QString &errorMsg)
+{
+    fprintf(stderr, "%s\n", errorMsg.toLocal8Bit().data());
+    QByteArray utf8ErrorMsg = errorMsg.toUtf8();
+    d.result = 3; // Error with msg
+    write(d.fd[1], &d.result, 1);
+    int l = utf8ErrorMsg.length();
+    write(d.fd[1], &l, sizeof(int));
+    write(d.fd[1], utf8ErrorMsg.data(), l);
+    close(d.fd[1]);
+    exit(255);
+}
+
 static pid_t launch(int argc, const char *_name, const char *args,
-                    const char *cwd = 0, int envc = 0, const char *envs = 0,
+                    const char *cwd = nullptr, int envc = 0, const char *envs = nullptr,
                     bool reset_env = false,
-                    const char *tty = 0, bool avoid_loops = false,
+                    const char *tty = nullptr, bool avoid_loops = false,
                     const char *startup_id_str = "0")  // krazy:exclude=doublequote_chars
 {
     QString lib;
@@ -557,7 +530,7 @@ static pid_t launch(int argc, const char *_name, const char *args,
         return d.fork;
     }
 
-#if HAVE_X11
+#if HAVE_XCB
     KStartupInfoId startup_id;
     startup_id.initId(startup_id_str);
     if (!startup_id.none()) {
@@ -566,11 +539,8 @@ static pid_t launch(int argc, const char *_name, const char *args,
 #endif
     // find out this path before forking, doing it afterwards
     // crashes on some platforms, notably OSX
-#ifdef Q_OS_MAC
-    const QString bundlepath = QStandardPaths::findExecutable(QFile::decodeName(execpath));
-#endif
 
-    d.errorMsg = 0;
+    d.errorMsg = nullptr;
     d.fork = fork();
     switch (d.fork) {
     case -1:
@@ -631,14 +601,6 @@ static pid_t launch(int argc, const char *_name, const char *args,
             QByteArray procTitle;
             d.argv = (char **) malloc(sizeof(char *) * (argc + 1));
             d.argv[0] = (char *) _name;
-#ifdef Q_OS_MAC
-            QString argvexe = QStandardPaths::findExecutable(QString::fromLatin1(d.argv[0]));
-            if (!argvexe.isEmpty()) {
-                QByteArray cstr = argvexe.toLocal8Bit();
-                // qDebug() << "kdeinit5: launch() setting argv: " << cstr.data();
-                d.argv[0] = strdup(cstr.data());
-            }
-#endif
             for (int i = 1;  i < argc; i++) {
                 d.argv[i] = (char *) args;
                 procTitle += ' ';
@@ -648,7 +610,7 @@ static pid_t launch(int argc, const char *_name, const char *args,
                 }
                 args++;
             }
-            d.argv[argc] = 0;
+            d.argv[argc] = nullptr;
 
 #ifndef SKIP_PROCTITLE
             /** Give the process a new name **/
@@ -671,25 +633,30 @@ static pid_t launch(int argc, const char *_name, const char *args,
             exitWithErrorMsg(errorMsg);
         }
 
-        if (!qgetenv("KDE_IS_PRELINKED").isEmpty() && !execpath.isEmpty()) {
+        if (!qEnvironmentVariableIsEmpty("KDE_IS_PRELINKED") && !execpath.isEmpty()) {
             libpath.truncate(0);
         }
 
         QLibrary l(libpath);
 
         if (!libpath.isEmpty()) {
-            if (!l.load()) {
-                if (libpath_relative) {
-                    // NB: Because Qt makes the actual dlopen() call, the
-                    //     RUNPATH of kdeinit is *not* respected - see
-                    //     https://sourceware.org/bugzilla/show_bug.cgi?id=13945
-                    //     - so we try hacking it in ourselves
-                    QString install_lib_dir = QFile::decodeName(
-                            CMAKE_INSTALL_PREFIX "/" LIB_INSTALL_DIR "/");
-                    libpath = install_lib_dir + libpath;
+            if (libpath_relative) {
+                // NB: Because Qt makes the actual dlopen() call, the
+                //     RUNPATH of kdeinit is *not* respected - see
+                //     https://sourceware.org/bugzilla/show_bug.cgi?id=13945
+                //     - so we try hacking it in ourselves
+                QString install_lib_dir = QFile::decodeName(
+                        CMAKE_INSTALL_PREFIX "/" LIB_INSTALL_DIR "/");
+                QString orig_libpath = libpath;
+                libpath = install_lib_dir + libpath;
+                l.setFileName(libpath);
+                if (!l.load()) {
+                    libpath = orig_libpath;
                     l.setFileName(libpath);
                     l.load();
                 }
+            } else {
+                l.load();
             }
             if (!l.isLoaded()) {
                 QString ltdlError(l.errorString());
@@ -715,11 +682,6 @@ static pid_t launch(int argc, const char *_name, const char *args,
             setup_tty(tty);
 
             QByteArray executable = execpath;
-#ifdef Q_OS_MAC
-            if (!bundlepath.isEmpty()) {
-                executable = QFile::encodeName(bundlepath);
-            }
-#endif
 
             if (!executable.isEmpty()) {
                 execvp(executable.constData(), d.argv);
@@ -816,7 +778,7 @@ static pid_t launch(int argc, const char *_name, const char *args,
         }
         close(d.fd[0]);
     }
-#if HAVE_X11
+#if HAVE_XCB
     if (!startup_id.none()) {
         if (d.fork && d.result == 0) { // launched successfully
             complete_startup_info(startup_id, d.fork);
@@ -827,6 +789,7 @@ static pid_t launch(int argc, const char *_name, const char *args,
 #endif
     return d.fork;
 }
+#endif // !Q_OS_OSX
 
 extern "C" {
 
@@ -875,7 +838,7 @@ static void init_signals()
     act.sa_handler = sig_child_handler;
     sigemptyset(&(act.sa_mask));
     sigaddset(&(act.sa_mask), SIGCHLD);
-    sigprocmask(SIG_UNBLOCK, &(act.sa_mask), 0L);
+    sigprocmask(SIG_UNBLOCK, &(act.sa_mask), nullptr);
     act.sa_flags = SA_NOCLDSTOP;
 
     // CC: take care of SunOS which automatically restarts interrupted system
@@ -884,14 +847,14 @@ static void init_signals()
 #ifdef SA_RESTART
     act.sa_flags |= SA_RESTART;
 #endif
-    sigaction(SIGCHLD, &act, 0L);
+    sigaction(SIGCHLD, &act, nullptr);
 
     act.sa_handler = SIG_IGN;
     sigemptyset(&(act.sa_mask));
     sigaddset(&(act.sa_mask), SIGPIPE);
-    sigprocmask(SIG_UNBLOCK, &(act.sa_mask), 0L);
+    sigprocmask(SIG_UNBLOCK, &(act.sa_mask), nullptr);
     act.sa_flags = 0;
-    sigaction(SIGPIPE, &act, 0L);
+    sigaction(SIGPIPE, &act, nullptr);
 }
 
 static void init_kdeinit_socket()
@@ -942,7 +905,7 @@ static void init_kdeinit_socket()
             exit(255);
         }
         server.sun_family = AF_UNIX;
-        strcpy(server.sun_path, sock_file);
+        qstrncpy(server.sun_path, sock_file, sizeof(server.sun_path));
         socklen = sizeof(server);
 
         if (connect(s, (struct sockaddr *)&server, socklen) == 0) {
@@ -984,7 +947,7 @@ static void init_kdeinit_socket()
         socklen = sizeof(sa);
         memset(&sa, 0, socklen);
         sa.sun_family = AF_UNIX;
-        strcpy(sa.sun_path, sock_file);
+        qstrncpy(sa.sun_path, sock_file, sizeof(sa.sun_path));
         if (bind(d.wrapper, (struct sockaddr *)&sa, socklen) != 0) {
             if (max_tries == 0) {
                 perror("kdeinit5: Aborting. bind() failed");
@@ -1046,7 +1009,7 @@ static void start_klauncher()
     char args[32];
     strcpy(args, "--fd=");
     sprintf(args + 5, "%d", d.launcher[1]);
-    d.launcher_pid = launch(2, "klauncher", args);
+    d.launcher_pid = launch(2, CMAKE_INSTALL_FULL_LIBEXECDIR_KF5 "/klauncher", args);
     close(d.launcher[1]);
 #ifndef NDEBUG
     fprintf(stderr, "kdeinit5: Launched KLauncher, pid = %ld, result = %d\n",
@@ -1086,14 +1049,15 @@ static bool handle_launcher_request(int sock, const char *who)
     (void)who; // for NDEBUG
 
     klauncher_header request_header;
-    char *request_data = 0L;
+    char *request_data = nullptr;
     int result = read_socket(sock, (char *) &request_header, sizeof(request_header));
     if (result != 0) {
         return false;
     }
 
     if (request_header.arg_length != 0) {
-        request_data = (char *) malloc(request_header.arg_length);
+        request_data = (char *) malloc(request_header.arg_length + 1);
+        request_data[request_header.arg_length] = '\0';
 
         result = read_socket(sock, request_data, request_header.arg_length);
         if (result != 0) {
@@ -1119,10 +1083,10 @@ static bool handle_launcher_request(int sock, const char *who)
         int argc = l;
         const char *name = request_data + sizeof(long);
         const char *args = name + strlen(name) + 1;
-        const char *cwd = 0;
+        const char *cwd = nullptr;
         int envc = 0;
-        const char *envs = 0;
-        const char *tty = 0;
+        const char *envs = nullptr;
+        const char *tty = nullptr;
         int avoid_loops = 0;
         const char *startup_id_str = "0"; // krazy:exclude=doublequote_chars
 
@@ -1184,25 +1148,28 @@ static bool handle_launcher_request(int sock, const char *who)
             return true; // sure?
         }
 
+#if HAVE_X11 || HAVE_XCB
         // support for the old a bit broken way of setting DISPLAY for multihead
-        QByteArray olddisplay = qgetenv(displayEnvVarName().constData());
+        QByteArray olddisplay = qgetenv(displayEnvVarName_c());
         QByteArray kdedisplay = qgetenv("KDE_DISPLAY");
         bool reset_display = (! olddisplay.isEmpty() &&
                               ! kdedisplay.isEmpty() &&
                               olddisplay != kdedisplay);
 
         if (reset_display) {
-            qputenv(displayEnvVarName().constData(), kdedisplay);
+            qputenv(displayEnvVarName_c(), kdedisplay);
         }
-
+#endif
         pid = launch(argc, name, args, cwd, envc, envs,
                      request_header.cmd == LAUNCHER_SHELL || request_header.cmd == LAUNCHER_KWRAPPER,
                      tty, avoid_loops, startup_id_str);
 
+#if HAVE_X11 || HAVE_XCB
         if (reset_display) {
             unsetenv("KDE_DISPLAY");
-            qputenv(displayEnvVarName().constData(), olddisplay);
+            qputenv(displayEnvVarName_c(), olddisplay);
         }
+#endif
 
         if (pid && (d.result == 0)) {
             response_header.cmd = LAUNCHER_OK;
@@ -1254,7 +1221,7 @@ static bool handle_launcher_request(int sock, const char *who)
         fprintf(stderr, "kdeinit5: terminate KDE.\n");
 #endif
 #if HAVE_X11
-        kdeinit_xio_errhandler(0L);
+        kdeinit_xio_errhandler(nullptr);
 #endif
     } else if (request_header.cmd == LAUNCHER_TERMINATE_KDEINIT) {
 #ifndef NDEBUG
@@ -1362,7 +1329,7 @@ static void handle_requests(pid_t waitForPid)
         }
 #endif
 
-        result = select(max_sock, &rd_set, &wr_set, &e_set, 0);
+        result = select(max_sock, &rd_set, &wr_set, &e_set, nullptr);
         if (result < 0) {
             if (errno == EINTR || errno == EAGAIN) {
                 continue;
@@ -1397,7 +1364,7 @@ static void handle_requests(pid_t waitForPid)
 #if HAVE_X11
         /* Look for incoming X11 events */
         if (X11fd >= 0 && FD_ISSET(X11fd, &rd_set)) {
-            if (X11display != 0) {
+            if (X11display != nullptr) {
                 XEvent event_return;
                 while (XPending(X11display)) {
                     XNextEvent(X11display, &event_return);
@@ -1411,12 +1378,11 @@ static void handle_requests(pid_t waitForPid)
 static void generate_socket_name()
 {
 
-    QByteArray display = qgetenv(displayEnvVarName().constData());
+#if HAVE_X11 || HAVE_XCB // qt5: see displayEnvVarName_c()
+    QByteArray display = qgetenv(displayEnvVarName_c());
     if (display.isEmpty()) {
-#if HAVE_X11 // qt5: see displayEnvVarName()
-        fprintf(stderr, "kdeinit5: Aborting. $%s is not set. \n", displayEnvVarName().constData());
+        fprintf(stderr, "kdeinit5: Aborting. $%s is not set. \n", displayEnvVarName_c());
         exit(255);
-#endif
     }
     int i;
     if ((i = display.lastIndexOf('.')) > display.lastIndexOf(':') && i >= 0) {
@@ -1425,12 +1391,17 @@ static void generate_socket_name()
 
     display.replace(':', '_');
 #ifdef __APPLE__
+    // not entirely impossible, so let's leave it
     display.replace('/', '_');
+#endif
+#else
+    // not using a DISPLAY variable; use an empty string instead
+    QByteArray display = "";
 #endif
     // WARNING, if you change the socket name, adjust kwrapper too
-    const QString socketFileName = QString::fromLatin1("kdeinit5_%1").arg(QLatin1String(display));
-    QByteArray socketName = QFile::encodeName(QStandardPaths::writableLocation(QStandardPaths::RuntimeLocation) + QLatin1Char('/') + socketFileName);
-    if (socketName.length() >= MAX_SOCK_FILE) {
+    const QString socketFileName = QStringLiteral("kdeinit5_%1").arg(QLatin1String(display));
+    const QByteArray socketName = QFile::encodeName(QStandardPaths::writableLocation(QStandardPaths::RuntimeLocation) + QLatin1Char('/') + socketFileName);
+    if (static_cast<unsigned>(socketName.length()) >= MAX_SOCK_FILE) {
         fprintf(stderr, "kdeinit5: Aborting. Socket name will be too long:\n");
         fprintf(stderr, "         '%s'\n", socketName.data());
         exit(255);
@@ -1507,9 +1478,7 @@ int kdeinit_x_errhandler(Display *dpy, XErrorEvent *err)
 #endif
     return 0;
 }
-#endif
 
-#if HAVE_X11
 // needs to be done sooner than initXconnection() because of also opening
 // another X connection for startup notification purposes
 static void setupX()
@@ -1532,8 +1501,8 @@ static void setupX()
         as well, point XAUTHORITY there and never remove the file (except for possible
         tmp cleanup).
     */
-    if (!qgetenv("XAUTHORITY").isEmpty()) {
-        QByteArray display = qgetenv(displayEnvVarName().constData());
+    if (!qEnvironmentVariableIsEmpty("XAUTHORITY")) {
+        QByteArray display = qgetenv(displayEnvVarName_c());
         int i;
         if ((i = display.lastIndexOf('.')) > display.lastIndexOf(':') && i >= 0) {
             display.truncate(i);
@@ -1543,23 +1512,26 @@ static void setupX()
         display.replace('/', '_');
 #endif
         QString xauth = QDir::tempPath() + QLatin1String("/xauth-")
-                        + QString::number(getuid()) + QLatin1String("-") + QString::fromLocal8Bit(display);
+                        + QString::number(getuid()) + QLatin1Char('-') + QString::fromLocal8Bit(display);
         QSaveFile xauthfile(xauth);
         QFile xauthfrom(QFile::decodeName(qgetenv("XAUTHORITY")));
+        // Set umask to make sure the file permissions of xauthfile are correct
+        mode_t oldMask = umask(S_IRGRP | S_IROTH | S_IWGRP | S_IWOTH);
         if (!xauthfrom.open(QFile::ReadOnly) || !xauthfile.open(QFile::WriteOnly)
                 || xauthfile.write(xauthfrom.readAll()) != xauthfrom.size() || !xauthfile.commit()) {
             // error
         } else {
             qputenv("XAUTHORITY", QFile::encodeName(xauth));
         }
+        umask(oldMask);
     }
 }
 
 // Borrowed from kdebase/kaudio/kaudioserver.cpp
 static int initXconnection()
 {
-    X11display = XOpenDisplay(NULL);
-    if (X11display != 0) {
+    X11display = XOpenDisplay(nullptr);
+    if (X11display != nullptr) {
         XCreateSimpleWindow(X11display, DefaultRootWindow(X11display), 0, 0, 1, 1, \
                             0,
                             BlackPixelOfScreen(DefaultScreenOfDisplay(X11display)),
@@ -1579,6 +1551,7 @@ static int initXconnection()
 }
 #endif
 
+#ifndef Q_OS_OSX
 // Find a shared lib in the lib dir, e.g. libkio.so.
 // Completely unrelated to plugins.
 static QString findSharedLib(const QString &lib)
@@ -1590,64 +1563,13 @@ static QString findSharedLib(const QString &lib)
     // We could also look in LD_LIBRARY_PATH, but really, who installs the main libs in different prefixes?
     return QString();
 }
-
-#ifdef Q_OS_MAC
-/**
- Calling CoreFoundation APIs (which is unavoidable in Qt/Mac) has always had issues
- on Mac OS X, but as of 10.5 is explicitly disallowed with an exception.  As a
- result, in the case where we would normally fork and then dlopen code, or continue
- to run other code, we must now fork-and-exec.
-
- See "CoreFoundation and fork()" at http://developer.apple.com/releasenotes/CoreFoundation/CoreFoundation.html
-*/
-// Copied from kkernel_mac.cpp
-void
-mac_fork_and_reexec_self()
-{
-    int argc = *_NSGetArgc();
-    char **argv = *_NSGetArgv();
-    char *newargv[argc + 2];
-    char progname[PATH_MAX];
-    uint32_t buflen = PATH_MAX;
-    _NSGetExecutablePath(progname, &buflen);
-    bool found_psn = false;
-
-    for (int i = 0; i < argc; i++) {
-        newargv[i] = argv[i];
-    }
-
-    newargv[argc] = "--nofork";
-    newargv[argc + 1] = NULL;
-
-    int x_fork_result = fork();
-    switch (x_fork_result) {
-
-    case -1:
-#ifndef NDEBUG
-        fprintf(stderr, "Mac OS X workaround fork() failed!\n");
-#endif
-        ::_exit(255);
-        break;
-
-    case 0:
-        // Child
-        execvp(progname, newargv);
-        break;
-
-    default:
-        // Parent
-        _exit(0);
-        break;
-
-    }
-}
 #endif
 
 extern "C" {
 
     static void secondary_child_handler(int)
     {
-        waitpid(-1, 0, WNOHANG);
+        waitpid(-1, nullptr, WNOHANG);
     }
 
 }
@@ -1719,7 +1641,7 @@ int main(int argc, char **argv)
     (void)dup2(2, 1);
 
     if (do_fork) {
-#ifdef Q_OS_MAC
+#ifdef Q_OS_OSX
         mac_fork_and_reexec_self();
 #else
         if (pipe(d.initpipe) != 0) {
@@ -1770,7 +1692,7 @@ int main(int argc, char **argv)
     d.accepted_fd = -1;
     d.debug_wait = false;
     d.launcher_ok = false;
-    children = NULL;
+    children = nullptr;
     init_signals();
 #if HAVE_X11
     setupX();
@@ -1784,8 +1706,8 @@ int main(int argc, char **argv)
          */
         init_kdeinit_socket();
     }
-#ifdef Q_OS_UNIX
-    if (!d.suicide && qgetenv("KDE_IS_PRELINKED").isEmpty()) {
+#if defined(Q_OS_UNIX) && !defined(Q_OS_OSX)
+    if (!d.suicide && qEnvironmentVariableIsEmpty("KDE_IS_PRELINKED")) {
         const int extrasCount = sizeof(extra_libs) / sizeof(extra_libs[0]);
         for (int i = 0; i < extrasCount; i++) {
             const QString extra = findSharedLib(QString::fromLatin1(extra_libs[i]));
@@ -1818,14 +1740,14 @@ int main(int argc, char **argv)
         if (XSupportsLocale()) {
             // Similar to QApplication::create_xim()
             // but we need to use our own display
-            XOpenIM(X11display, 0, 0, 0);
+            XOpenIM(X11display, nullptr, nullptr, nullptr);
         }
 #endif
     }
 
     if (launch_kded) {
         qputenv("KDED_STARTED_BY_KDEINIT", "1");
-        pid = launch(1, KDED_EXENAME, 0);
+        pid = launch(1, KDED_EXENAME, nullptr);
         unsetenv("KDED_STARTED_BY_KDEINIT");
 #ifndef NDEBUG
         fprintf(stderr, "kdeinit5: Launched KDED, pid = %ld result = %d\n", (long) pid, d.result);
@@ -1837,7 +1759,7 @@ int main(int argc, char **argv)
 
     for (int i = 1; i < argc; i++) {
         if (safe_argv[i][0] == '+') {
-            pid = launch(1, safe_argv[i] + 1, 0);
+            pid = launch(1, safe_argv[i] + 1, nullptr);
 #ifndef NDEBUG
             fprintf(stderr, "kdeinit5: Launched '%s', pid = %ld result = %d\n", safe_argv[i] + 1, (long) pid, d.result);
 #endif
@@ -1849,7 +1771,7 @@ int main(int argc, char **argv)
                   ) {
             // Ignore
         } else {
-            pid = launch(1, safe_argv[i], 0);
+            pid = launch(1, safe_argv[i], nullptr);
 #ifndef NDEBUG
             fprintf(stderr, "kdeinit5: Launched '%s', pid = %ld result = %d\n", safe_argv[i], (long) pid, d.result);
 #endif
